"""Functions to create the PiLibre screensaver."""

import time

from rich.console import Console
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.text import Text

from pilibre.console import console
from pilibre.screensaver._background import construct_screensaver
from pilibre.screensaver._center import align_logo
from pilibre.screensaver._clock import format_datetime_title
from pilibre.screensaver._logo import LOGO


def create_screen(logo: Text, console: Console) -> Layout:
    """Creates a static screensaver.

    Args:
        logo (Text): Text to display.
        console (Console): Console instance.

    Returns:
        Layout: Screensaver to display.
    """
    date_time = format_datetime_title("America/Denver")

    aligned_logo = align_logo(logo, console.width, console.height)
    stars = construct_screensaver(aligned_logo, console.width)

    screen = Layout(
        Panel(
            stars,
            title=date_time,
            title_align="left",
            border_style="black",
        )
    )

    return screen


def run_screensaver(seconds: int) -> None:
    """Runs the screensaver, for the supplied amount of seconds.

    Args:
        seconds (int): Time to run screensaver, in seconds.

    Returns:
        None
    """
    with Live(create_screen(LOGO, console), refresh_per_second=4) as live:
        for _ in range(seconds):
            time.sleep(1)
            live.update(create_screen(LOGO, console))



"""This module loads the PiLibre configuration."""


import os
from pathlib import Path

_FILE_NAME = "pilibre.toml"
_DIR_NAME = "pilibre"


def _file_exists(file: Path) -> bool:
    file_exists = file.exists() and file.is_file()
    return file_exists


def _get_config_file(directory: Path) -> Path | None:
    config_file = directory / _FILE_NAME

    if config_file.exists() and config_file.is_file():
        return config_file

    return None


def find_config() -> Path:
    """Get the path to the user configuration file.

    Log all the config files that are found.

    Raises:
        FileNotFoundError: Raised if no configuration file is found.

    Returns:
        Path: Path to configuration file.
    """
    xdg_config = Path(os.environ["XDG_CONFIG_HOME"]) / _DIR_NAME
    dot_config = Path().home() / ".config" / _DIR_NAME

    configs = []

    if xdg_config_file := _get_config_file(xdg_config):
        configs.append(xdg_config_file)

    if dot_config_file := _get_config_file(dot_config):
        configs.append(dot_config_file)

    if not configs:
        raise FileNotFoundError("No configuration file found.")

    # Only return the first found
    return configs[0]



"""Main package entry point."""


def main() -> None:
    """Main function. Runs PiLibre."""
    # from pilibre.screensaver.screensaver import run_screensaver

    # run_screensaver(20)
    pass


if __name__ == "__main__":
    main()



import random

from rich.text import Text

from pilibre._theme import theme


def _replace_single_char(text: Text, char: str, index: int) -> Text:
    """Replaces a single character in a `Text` object, at the index.

    Note that this will work with strings, too. In the context of this
    application, I will only be passing `Text`, to keep color or other
    `Rich` formatting.

    Args:
        text (Text): Text to replace within.
        char (str): Character to replace.
        index (int): Index at which to replace.

    Returns
        Text: Original text with the character replaced.
    """
    return text[:index] + char + text[index + 1 :]


def _check_if_empty_char(text: Text, index: int) -> bool:
    """Check if a character at the supplied index is a space character.

    Args:
        text (Text): Text to check within.
        index (int): Index at which to check for a space character.

    Returns
        bool: True, if the character is a space character.
    """
    text_as_string = text.plain
    char = text_as_string[index]
    return char.isspace()


def _replace_random_spaces(text: Text, char: str, num: int) -> Text:
    """Replace `num` random whitespaces in `text`, with `char`.

    Args:
        text (Text): Text to replace within.
        char (str): Character to replace with.
        num (int): Number of replacements to perform.

    Returns
        Text: Text
    """
    i = 0
    while i < num:
        index = random.randint(0, len(text) - 1)

        # If we randomly grab a non-whitespace character, try again
        if not _check_if_empty_char(text, index):
            continue

        text = _replace_single_char(text, char, index)
        i += 1

    return text


def construct_screensaver(foreground: Text, width: int) -> Text:
    """Constructs the starry background text for the screensaver.

    Args:
        foreground (Text): Foreground text.
        width (int): Maximum line width.

    Returns
        Text: Starry background, as Text.
    """
    new_line = Text("\n")
    lines = []

    for line in foreground.split():
        # line = Text(" " * width)
        line = _replace_random_spaces(line, "⋆", 10)
        line = _replace_random_spaces(line, "✷", 2)
        line = _replace_random_spaces(line, "∙", 4)
        line = _replace_random_spaces(line, "✦", 8)
        line = _replace_random_spaces(line, "✧", 2)

        line.set_length(width - 4)

        lines.append(line)

    background = new_line.join(lines)

    background.highlight_words("⋆", style=theme.styles["star.far"])
    background.highlight_words("✷", style=theme.styles["star.yellow"])
    background.highlight_words("∙", style=theme.styles["star.blue"])
    background.highlight_words("✦", style=theme.styles["star.near"])
    background.highlight_words("✧", style=theme.styles["star.hallow"])

    return background



from rich.text import Text


def _center_horizontally(logo: Text, width: int) -> Text:
    # Round down, -52 for logo width and panel characters
    num_spaces = (width - 52) // 2
    padding = Text((num_spaces * " "))

    lines = logo.split()
    padded_lines: list[Text] = []

    for line in lines:
        centered_line = padding + line + padding
        centered_line.set_length((width - 2))

        padded_lines.append(centered_line)

    centered_logo = Text("\n").join(padded_lines)

    return centered_logo


def _center_vertically(logo: Text, width: int, height: int) -> Text:
    blank_line = Text(" " * width + "\n")

    # Even number of lines, logo is 6 lines tall
    if (height - 6) % 2 == 0:
        pad_num = (height - 6) // 2

        padding = blank_line.copy()
        if pad_num == 1:
            padding.append_text(logo)
            return padding

        for _ in range(pad_num - 2):
            padding.append_text(blank_line)

        padding_bottom = padding.copy()
        padding.append_text(logo)
        padding.append_text(padding_bottom)

        return padding

    # Odd number of lines
    else:
        pad_num = height - 6

        top_num = ((pad_num - 1) // 2) + 1
        bottom_num = (pad_num - 1) // 2

        padding_top = blank_line.copy()
        padding_bottom = blank_line.copy()

        if pad_num == 1:
            padding_top.append_text(logo)
            return padding_top

        # Subtract 2: 1 because blank_line has \n, another for the panel
        for _ in range(top_num - 2):
            padding_top.append_text(blank_line)

        for _ in range(bottom_num - 2):
            padding_bottom.append_text(blank_line)

        padding_top.append_text(logo)
        padding_top.append_text(padding_bottom)

        return padding_top


def align_logo(logo: Text, width: int, height: int) -> Text:
    """Manually construct the centered logo.

    Note that this effectively does what `Rich.Align` does, but keeps
    the logo as `Text`.

    Args:
        logo (Text): Logo text.
        width (int): Terminal width.
        height (int): Terminal height

    Returns:
        Text: Centered logo.
    """
    horizontal = _center_horizontally(logo, width)
    vertical = _center_vertically(horizontal, width, height)

    return vertical



"""Provides the `console` to all submodules.

Attributes:
    console (Console): Global console varaible.
"""

from rich.console import Console

from pilibre._theme import theme

console = Console(theme=theme)



from rich.text import Text

from pilibre._theme import theme

# 50 characters, by 6 lines
LOGO_UNFORMATTED = [
    "██████╗ ██╗██╗     ██╗██████╗ ██████╗ ███████╗",
    "██╔══██╗╚═╝██║     ╚═╝██╔══██╗██╔══██╗██╔════╝",
    "██████╔╝██║██║     ██║██████╔╝██████╔╝█████╗  ",
    "██╔═══╝ ██║██║     ██║██╔══██╗██╔══██╗██╔══╝  ",
    "██║     ██║███████╗██║██████╔╝██║  ██║███████╗",
    "╚═╝     ╚═╝╚══════╝╚═╝╚═════╝ ╚═╝  ╚═╝╚══════╝",
]


LINE_COLORS = [
    theme.styles["logo.lighter"],
    theme.styles["logo.light"],
    theme.styles["logo.neutral"],
    theme.styles["logo.dark"],
    theme.styles["logo.darker"],
    "",
]
SHADOW_COLOR = theme.styles["logo.shadow"]


def _paint_lines(logo: list[str]) -> Text:
    """Apply a style to each line in the PiLibre logo.

    Args:
        logo (list[str]): PiLibre logo.

    Returns:
        Text: Colored logo.
    """
    text_logo = Text()
    for i, line in enumerate(logo):
        painted_line = Text.assemble((line, f"{LINE_COLORS[i]}"), "\n")
        text_logo.append(painted_line)

    return text_logo


def construct_logo(logo: list[str]) -> Text:
    """Colorize the PiLibre logo.

    This applies `_paint_lines`, which also colors the shadow
    characters. Calls `Text.highlight_regex` after to correctly color
    the shadow characters.

    Args:
        logo (list[str]): PiLibre logo.

    Returns:
        Text: Finalized, colored logo.
    """
    painted_logo = _paint_lines(logo)
    painted_logo.highlight_regex(r"(?P<shadow>[╔╗╚╝═║])", style=SHADOW_COLOR)

    return painted_logo


LOGO = construct_logo(LOGO_UNFORMATTED)



from datetime import datetime
from zoneinfo import ZoneInfo, ZoneInfoNotFoundError

from rich.text import Text


def format_datetime_title(timezone: str) -> Text:
    """Format the simple clock title.

    If `timezone` is not found in the IANA database
    (see: https://docs.python.org/3/library/zoneinfo.html and
    https://en.wikipedia.org/wiki/List_of_tz_database_time_zones), the
    function returns 'Invalid timezone' as `Text`.

    Args:
        timezone (str): Local timezone.

    Returns:
        Text: Formatted date and local time.
    """
    try:
        tz = ZoneInfo(timezone)
    except ZoneInfoNotFoundError:
        clock = Text()
        clock.append("Invalid timezone.", style="red")
        return clock

    dt = datetime.now(tz)

    time = dt.strftime("%H:%M")
    date = dt.strftime("%A, %B %d | ")

    clock = Text()
    clock.append(date, style="white")
    clock.append(time, style="white")

    return clock



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pilibre', 'pilibre.config', 'pilibre.graph', 'pilibre.screensaver']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.23.3,<0.24.0', 'rich>=13.0.1,<14.0.0']

entry_points = \
{'console_scripts': ['pilibre = pilibre.main:main']}

setup_kwargs = {
    'name': 'pilibre',
    'version': '0.1.0',
    'description': 'Display hardware metrics from LibreHardwareMonitor, intended to be run on a small screen on a RaspberryPi.',
    'long_description': '<p align="center">\n  <img alt="PiLibre" src="docs/assets/logo.svg"/>\n</p>\n\n<div align="center">\n    <img alt="PyPI" src="https://img.shields.io/pypi/v/pilibre?style=for-the-badge">\n    <img alt="PyPI - Python Version" src="https://img.shields.io/pypi/pyversions/pilibre?style=for-the-badge">\n    <img alt="GitHub" src="https://img.shields.io/github/license/acbuie/pilibre?style=for-the-badge">\n</div>\n\n<div align="center">\n    <img alt="GitHub Repo stars" src="https://img.shields.io/github/stars/acbuie/pilibre?style=for-the-badge">\n    <a href="https://github.com/acbuie">\n        <img alt="GitHub Profile" src="https://img.shields.io/static/v1?label=&message=Profile&style=for-the-badge&logo=github&labelColor=grey">\n    </a>\n</div>\n\nA small Python script for displaying hardware metrics. Metrics are pulled from `LibreHardwareMonitor`, and requires it to be installed on your PC and in server mode. It is designed to be run on a small computer with a small display. I intend to use a Raspberry Pi, but you could likely use something else.\n\n<!-- Some examples will go here! -->\n\n---\n\n## Installation\n\nPiLibre relies on `rich` and `httpx` on the display machine, and `LibreHardwareMonitor` on the machine from which you want to see metrics.\n\n- `rich`: For terminal display of the hardware metrics\n- `https`: To pull the JSON data served by `LibreHardwareMonitor`\n- `LibreHardwareMonitor`: For getting the hardware metrics of your computer.\n\n### Host Machine Setup\n\nThe host machine only needs to have `LibreHardwareMonitor` installed. The project can be found here: https://github.com/LibreHardwareMonitor/LibreHardwareMonitor and downloaded from here: https://github.com/LibreHardwareMonitor/LibreHardwareMonitor/releases.\n\nOnce installed:\n\n- Open the HTTP server via `Options -> HTTP Server`.\n- Take note of the IP address. This is the IP address of the local machine.\n- If you\'d like to change the default port, do so here.\n\n### Display Machine Setup\n\nThe machine you want to display metrics on needs to have the `PiLibre` application installed. The easiest way is to use `pipx`, but manual instructions are also included.\n\n#### pipx\n\n`pipx` is recommended for the simplest installation. This will install `PiLibre` into its own virtual environment, along with any dependencies.\n\nInstall `pipx` from here: https://github.com/pypa/pipx#install-pipx.\n\n```shell\npipx install git+https://github.com/acbuie/pilibre.git\n```\n\n#### Manually\n\nIf you know what you\'re doing, you can install the package via `Git` and run it manually, with `python -m src/pilibre`. As always, a virtual environment is recommended, so the requirements don\'t get installed into the system python. Runtime dependencies can be installed with `python -m pip install requirements.txt`.\n\nFirst, clone the project into a new directory.\n\n```shell\nmkdir pilibre\ncd pilibre\ngit clone https://github.com/acbuie/pilibre.git\n```\n\nOnce installed, create and activate a python virtual environment. Read about python virtual environments here: https://docs.python.org/3/tutorial/venv.html.\n\nThen, install the dependencies.\n\n```shell\npython -m pip install requirements.txt\n```\n\n## Usage\n\nUsage is very simple. Once the HTTP server is running on the host machine, simply specify the IP address and port in the config file and run the project.\n',
    'author': 'acbuie',
    'author_email': 'aidancbuie@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/acbuie/pilibre',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)



from rich.theme import Theme

theme = Theme(
    {
        "logo.lighter": "#ebdbb2",
        "logo.light": "#d3c5a0",
        "logo.neutral": "#bcaf8e",
        "logo.dark": "#a4997c",
        "logo.darker": "#8d836a",
        "logo.shadow": "#3c3836",
        "star.near": "#ebdbb2",
        "star.hallow": "#ebdbb2",
        "star.yellow": "yellow",
        "star.blue": "blue",
        "star.far": "#8d836a",
    }
)
